use bitvec::vec::BitVec;
use flatten::Flattener;
use parser::Parser;
use scanner::Scanner;

mod expr;
mod flatten;
mod parser;
mod scanner;
mod token;

fn main() {
    let scanner = Scanner::new(
        "
$false=λtf.f
$omega=λa. (a a)


λa.a($omega(λbcde.d(bb)(λf.fce)))$false
",
    );

    let tokens = scanner.scan_tokens().unwrap();

    let mut parser = Parser::new(tokens);
    let (funcs, main) = parser.parse().unwrap();

    let mut flattener = Flattener::new(funcs);
    let flat = flattener.flatten(&main).unwrap();

    dbg!(&flat);

    let mut bv = BitVec::new();
    flat.to_blc(&mut bv);

    for x in bv.as_raw_slice() {
        print!("{}", *x as char);
    }
}
